import 'dart:convert';
import 'package:get/get.dart';
import 'package:namebase/namebase.dart';

import 'models/record.dart';
import 'models/response.dart';

/// ### Namebase API
/// - Handles HTTP requests to the Namebase API
/// - Provides a simple interface for accessing the Namebase API
/// **Required** : `API_KEY`, `API_SECRET`
class APIClient extends GetConnect {
  /// **Namebase** API key
  final String key;

  /// **Namebase** API secret
  final String secret;

  /// ### APIClient Constructor
  /// - Initializes the Namebase API.
  /// - Provides Methods to Handle Namebase API Requests
  /// **Required** : `API_KEY`, `API_SECRET`
  APIClient({required this.key, required this.secret});

  /// Base Client Properties
  final apiUrl = "https://www.namebase.io";

  /// Endpoint for Nameserver DNS Settings
  final apiDnsEndpoint = '/api/v0/dns/domains/snr/nameserver';

  /// Authorization Header
  String get authorization => base64Encode(utf8.encode("$key:$secret"));

  // Authorization Header Map
  Map<String, String> get authHeaders => {'Authorization': 'Basic $authorization', 'Accept': 'application/json', 'Content-Type': 'application/json'};

  /// #### `addRecords()`
  /// - Method Creates New Item(s) on DNS Table
  /// - Returns `true` if successful, `false` otherwise
  /// **Required** : `List<DNSRecord>`
  Future<bool> addRecords(List<DNSRecord> records) async {
    // Convert Records to Map List
    final List<Map<String, dynamic>> mappedRecords = records.map<Map<String, dynamic>>((e) => e.toMap()).toList();

    // Build Request
    var map = {'records': mappedRecords, 'deleteRecords': []};
    var body = jsonEncode(map);
    var resp = await put(apiUrl + apiDnsEndpoint, body, headers: authHeaders);
    return resp.body != null ? resp.body["success"] : true;
  }

  /// #### `deleteRecords()`
  /// - Method **DELETES** New Item(s) on DNS Table
  /// - Returns `true` if successful, `false` otherwise
  /// **Required** : `List<DNSRecord>`
  Future<bool> deleteRecords(List<DNSRecord> records) async {
    // Fetch records
    final data = await refresh();

    // Verify Records exist
    final validated = data.records.where((r) => records.any((gR) => gR.host == r.host));

    // Validate Length
    if (validated.length == records.length) {
      // Convert Records to Map List
      final List<Map<String, dynamic>> mappedRecords = records.map<Map<String, dynamic>>((e) => e.toMap()).toList();

      // Build Request
      var map = {'records': [], 'deleteRecords': mappedRecords};
      var body = jsonEncode(map);
      var resp = await put(apiUrl + apiDnsEndpoint, body, headers: authHeaders);
      return resp.body != null ? resp.body["success"] : true;
    }
    return false;
  }

  /// #### `refresh()`
  /// - Returns `NamebaseResponse` with *records* (`List<DNSRecord>`)
  ///   and *success* (`bool`)
  Future<NamebaseResponse> refresh() async {
    var resp = await get(apiUrl + apiDnsEndpoint, headers: authHeaders);
    return NamebaseResponse.fromResponse(resp);
  }

  /// #### `replaceRecord()`
  /// - Replaces a single record in the DNS Table
  /// - Returns `true` if successful
  /// - Returns `false` if the record does not exist
  /// **Required** : `DNSRecord`
  Future<bool> replaceRecord(DNSRecord record) async {
    // Fetch Data
    final data = await refresh();
    final oldRecord = data.records.firstWhere(
      (r) => r.equalsName(r.host),
      orElse: () => DNSRecord.blank(),
    );

    // Remove Old Record
    final hasDeletedOldRecord = await deleteRecords([oldRecord]);
    if (!hasDeletedOldRecord) {
      return false;
    }

    // Perform Add Request
    final hasAddedNewRecord = await addRecords([record]);
    if (!hasAddedNewRecord) {
      return false;
    }
    return true;
  }
}
