import 'package:get/get.dart';
import 'package:namebase/namebase.dart';
import 'package:namebase/src/models/result.dart';
import 'package:namebase/src/models/status.dart';
import 'record.dart';

// Restricted Username Values
const K_RESTRICTED_NAMES = ['elon', 'vitalik', 'prad', 'rishi', 'brax', 'vt', 'isa'];

/// Blocked Username Values
const K_BLOCKED_NAMES = ['root', 'admin', 'mail', 'auth', 'crypto', 'id', 'app', 'beta', 'alpha', 'code', 'ios', 'android', 'test', 'node', 'sonr'];

/// ##### `NamebaseResponse`
/// Class Returns `NameBase` Records on Refresh
class NamebaseResponse {
  final List<DNSRecord> records;
  final bool success;
  final bool isBlank;

  /// Checks if Name is not already Taken
  bool isNameAvailable(String n) => !records.any((r) => r.equalsName(n));

  /// Checks if Name not on Blocked List
  bool isNameUnblocked(String n) => !K_BLOCKED_NAMES.any((v) => v == n.toLowerCase());

  /// Checks if Name not on Restricted List
  bool isNameUnrestricted(String n) => !K_RESTRICTED_NAMES.any((v) => v == n.toLowerCase());

  /// Checks if Name Provided is Valid with ALL Checks
  bool isValidName(String n) => isNameAvailable(n) && isNameUnblocked(n) && isNameUnrestricted(n);

  NamebaseResponse({required this.success, this.isBlank = false, this.records = const []});

  /// Returns NamebaseResult from `GetConnect` Response
  factory NamebaseResponse.fromResponse(Response<dynamic> response) => NamebaseResponse(
        isBlank: false,
        success: response.body["success"] ?? false,
        records: List<DNSRecord>.from(response.body["records"].map((x) => DNSRecord.fromJson(x))),
      );

  /// Returns Blank Namebase Result
  factory NamebaseResponse.blank() => NamebaseResponse(success: false, isBlank: true);

  /// Checks with `NameCheckType` and Returns bool
  bool checkName(NameCheckType type, String name) {
    switch (type) {
      case NameCheckType.Available:
        return isNameAvailable(name);
      case NameCheckType.Unblocked:
        return isNameUnblocked(name);
      case NameCheckType.Unrestricted:
        return isNameUnrestricted(name);
      case NameCheckType.All:
        return isValidName(name);
      case NameCheckType.Blocked:
        return !isNameUnblocked(name);
      case NameCheckType.Restricted:
        return !isNameUnrestricted(name);
      case NameCheckType.Unavailable:
        return !isNameAvailable(name);
    }
  }

  /// Finds Matching Record with Username and Prefix
  NamebaseRecordResult hasName(String name, String prefix) {
    // Check Exists
    var exists = records.any((r) => r.equals(name, prefix));

    // Find Existing Record
    if (exists) {
      var record = records.firstWhere((r) => r.equals(name, prefix));

      // Return Result
      return NamebaseRecordResult(exists: true, record: record);
    }
    return NamebaseRecordResult(exists: false, record: DNSRecord.blank());
  }
}
